# encoding:utf-8
from django.db import models
from datetime import datetime, timedelta
from django.template import RequestContext, loader
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse
from django.core import validators
from django.core.validators import RegexValidator
import re


def es_numero(dato):
    """ validador numerico """
    # en un bloque aislamos la conversion a entero
    # y regresamos en un boleano si se lo gra o no se logra
    try:
        int(dato)
        return True
    except ValueError:
        return False


def es_cadena(dato):
    """ validador cadenas """
    # en un bloque aislamos la conversion a entero
    # y regresamos en un boleano si se lo gra o no se logra
    try:
        str(dato)
        return True
    except ValueError:
        return False


# validador de años de registro IFE, desde sus inicios hasta el actual
def valida_anio_registro(anio):
    """ Validador personalizado """
    # inicio de registros en el IFE
    anioRegistroInicio = 1990
    # año actual
    anioActual = datetime.now().year
    # verificamos que se trate de un numero
    if es_numero(anio):
        # verificamos el rango de años validos de registro
        if anio < anioRegistroInicio or anio > anioActual:
            # formamos la cadena de validacion
            raise ValidationError(u'%s no se encuentra en el rango %s - %s' % (anio, anioRegistroInicio, anioActual))
    else:
        # formamos la cadena si no es un numero entero
        raise ValidationError(u'%s no es un numero.' % anio)


def valida_folio(folio):
    """ Validador personalizado del folio de la credencial del IFE """
    #numerico  de longitud 13 con ceros a la izquierda y tipo de dato Char
    longitudFolio = len(folio)

    if longitudFolio != 13:
        raise ValidationError(
            u'el Folio "%s" debe tener una longitud de 13 caracteres, tiene %s' % (folio, longitudFolio))
    else:
        if re.match('[0-9]{13}', folio) == None:
            raise ValidationError(u'el Folio "%s" solo permite números' % (folio))


def valida_curp(curp):
    longitudCurp = len(curp)

    if longitudCurp != 18:
        raise ValidationError(u'La CURP "%s" debe tener una longitud de 18 caracteres, tiene %s' % (curp, longitudCurp))
    else:
        if re.match('[a-zA-Z]{4}[0-9]{6}[a-zA-Z]{6}[a-zA-Z0-9]{2}', curp) == None:
            raise ValidationError(u'la CURP "%s" no es correcto' % (curp))


def valida_clave_elector(clave):
    longitudClave = len(clave)
    if longitudClave != 18:
        raise ValidationError(
            u'la Clave de elector "%s" debe tener una longitud de 18 caracteres, tiene %s' % (clave, longitudClave))
    else:
        if re.match('[a-zA-Z]{6}[0-9]{8}[a-zA-Z]{1}[0-9]{3}', clave) == None:
            raise ValidationError(u'la Clave de Elector "%s" no es correcta' % (clave))


def valida_ocr(ocr):
    longitudOcr = len(ocr)
    if longitudOcr != 13:
        raise ValidationError(u'la OCR "%s" debe tener una longitud de 13 caracteres, tiene %s' % (ocr, longitudOcr))
    else:
        if re.match('[0-9]{13}', ocr) == None:
            raise ValidationError(u'la OCR "%s" solo permite números' % (ocr))


def valida_numeros(dato):
    if not es_numero(dato):
        raise ValidationError(u'"%s" no es un numero.' % dato)


def valida_cadenas(dato):
    if re.match('[a-zA-ZáéíóúÁÉÍÓÚ]+', dato) == None:
        raise ValidationError(u'"%s" no es una cadena.' % dato)


class Estado(models.Model):
    nombre = models.CharField(verbose_name='Estado', max_length=100)

    def __unicode__(self):
        return self.nombre

    class Meta:
        ordering = ['nombre']
        unique_together = ['nombre']


class Municipio(models.Model):
    estado = models.ForeignKey(Estado, verbose_name="Estado")
    nombre = models.CharField(verbose_name='Nombre', max_length=100)
    clave = models.CharField(max_length=4, verbose_name="Clave")

    def __unicode__(self):
        return self.nombre

    class Meta:
        ordering = ['nombre']
        unique_together = ['nombre', 'estado']


class Localidad(models.Model):
    municipio = models.ForeignKey(Municipio, verbose_name="Municipio")
    nombre = models.CharField('Localidad', max_length=100)
    listo = models.CharField(max_length=60)

    def __unicode__(self):
        return self.nombre

    class Meta:
        ordering = ['nombre']


class Domicilio(models.Model):
    calle = models.CharField('Calle', max_length=100, null=True, blank=True)
    numero = models.CharField('Numero', max_length=10, null=True, blank=True)
    codigo_postal = models.IntegerField('Codigo Postal', max_length=5, null=True, blank=True,validators=[valida_numeros])
    colonia = models.CharField('Colonia', max_length=500, validators=[valida_cadenas])
    longitud = models.IntegerField('Longitud', max_length=10, null=True, blank=True)
    latitud = models.IntegerField('Latitud', max_length=10, null=True, blank=True)
    localidad = models.ForeignKey(Localidad, verbose_name="Localidad")

    def __unicode__(self):
        return '%s' % self.calle



class Sexo(models.Model):
    tipo = models.CharField(max_length="10")

    def __unicode__(self):
        return '%s' % self.tipo


class Persona(models.Model):
    fecha_registro = models.DateTimeField(auto_now_add=True)
    nombre = models.CharField(max_length=50)
    apellido_paterno = models.CharField(max_length=30)
    apellido_materno = models.CharField(max_length=30)
    sexo = models.ForeignKey(Sexo, verbose_name="Sexo")
    fecha_nacimiento = models.DateField()
    domicilio = models.ForeignKey(Domicilio, verbose_name="Domicilio")
    curp = models.CharField('Curp',max_length=20,validators=[valida_curp])
    tipo = models.ManyToManyField("TipoPersona")
    activo = models.BooleanField(default=True)

    def __unicode__(self):
        return '%s %s %s' % (self.nombre, self.apellido_paterno, self.apellido_materno)


    class Meta:
        unique_together = ['domicilio', 'sexo','curp' ]

    def get_absolute_url(self):
        return reverse('add-personas')


class Credencial(models.Model):
    folio = models.CharField('Folio', max_length=13, validators=[valida_folio], null=True, blank=True)
    anio_registro = models.IntegerField('Año de Registro', validators=[valida_anio_registro])
    clave_elector = models.CharField('Clave de Elector', max_length=30, validators=[valida_clave_elector])
    ocr = models.CharField('OCR', max_length=20, validators=[valida_ocr], null=True, blank=True)
    persona=models.ForeignKey(Persona,verbose_name='Persona',null=True, blank=True)


    def __unicode__(self):
        return '%s' % self.clave_elector

    class Meta:
        unique_together = ['folio', 'clave_elector', 'ocr','persona']


class TipoPersona(models.Model):

    descripcion =models.CharField('Descripcion',max_length=30)
    fecha = models.DateTimeField('Fecha',auto_now_add=True)

    def __unicode__(self):
        return '%s' % self.descripcion